@@ -127,7 +127,7 @@ class LazyLocalContext(object) :
                 o, e = proc.communicate()
                 stdout = SPRetObj(o)
                 stderr = SPRetObj(e)
-            except:
+            except ValueError:
                 stdout = None
                 stderr = None
         return ret, stdout, stderr
